import logging
import os
from datetime import date

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
)
logger = logging.getLogger("scd_type2")

COLOURS = {
    "EOC": "\033[0m",  # End of colour
    "CYN": "\033[36m",  # Cyan
    "RED": "\033[91m",  # Red
    "MAG": "\033[35m",  # Magenta
}

today = date.today().strftime("%d/%m/%Y")



#! /usr/bin/env python3

"""
Script to compare source data with target data and get new DataFrame to be uploaded
to database, flagged as whether new (active=1) or old (active=0).
"""

from typing import Union

import pandas as pd

from scd_type2.utils import duplicate_warn, make_expired, make_new


def compare_dfs(df1: pd.DataFrame, df2: pd.DataFrame, idx: Union[list, str] = None):
    """
    Returns one DataFrame of rows to be created and one DataFrame of rows to be updated
    logic:
        if row of df1[idx] exists in df2[idx], then add to update DataFrame
        if row of df1[idx] does not exist in df2[idx], then add to create DataFrame
        otherwise, no action
    """
    if isinstance(idx, str):
        idx = [idx]
    elif idx is None:
        # All columns of both, which must be the same
        idx = df1.columns.to_list()

    sub_df1 = df1[idx]
    sub_df2 = df2[idx]

    # Duplicates removed, but may cause unexpected results
    if len(sub_df1) != len(sub_df1.drop_duplicates()):
        duplicate_warn()
        sub_df1.drop_duplicates(inplace=True)

    if len(sub_df2) != len(sub_df2.drop_duplicates()):
        duplicate_warn()
        sub_df2.drop_duplicates(inplace=True)

    merge_df = sub_df1.merge(sub_df2, indicator=True, how='left')
    end_idx = merge_df.copy(deep=True).loc[lambda x: x['_merge'] == 'both'].index

    end_df_old = df1.loc[end_idx]
    end_df = make_expired(end_df_old)

    end_df_new = df2.loc[end_idx]
    end_df_new = make_new(end_df_new)

    start_idx = merge_df.copy(deep=True).loc[lambda x: x['_merge'] == 'left_only'].index
    start_df = df1.loc[start_idx]
    start_df = make_new(start_df)

    start_df = pd.concat([start_df, end_df_new], ignore_index=True)

    return start_df, end_df



import warnings

import pandas as pd

from scd_type2.config import COLOURS, today


def duplicate_warn():
    warnings.warn(
        f"\n{COLOURS['RED']}Warning:{COLOURS['EOC']} There are multiple common records in DataFrame based on index.\n")


def make_expired(df: pd.DataFrame):
    if len(df) == 0:
        df = None
    else:
        df["active"] = 0
        df["end_date"] = today
    return df


def make_new(df: pd.DataFrame):
    if len(df) == 0:
        df = None
    else:
        df["active"] = 1
        df["start_date"] = today
    return df



import sys

from setuptools import setup, find_packages

CURRENT_PYTHON = sys.version_info[:2]
REQUIRED_PYTHON = (3, 9)

if CURRENT_PYTHON < REQUIRED_PYTHON:
    sys.stderr.write("""WARNING: This package has only been validated on Python 3.9+.""")

with open("README.md", "r") as fh:
    long_description = fh.read()

requires = [
    "numpy==1.22.4",
    "pandas==1.4.2",
    "python-dateutil==2.8.2",
    "pytz==2022.1",
    "six==1.16.0"
]

setup(
    name="scd_type2",
    version="0.1.0",
    author="Zac Clery",
    author_email="zacclery@gmail.com",
    description="Simplifies and unifies the logic for performing SCD type 2 data comparisons.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.9',
    install_requires=requires,
    keywords='scd type2 scd_type2 slowly changing dimensions slowly-changing-dimensions',
    entry_points={
        'console_scripts': [
            'scd_type2=scd_type2.__main__:main',
            'scd=scd_type2.__main__:main',
        ],
    },
)



from datetime import date

import pandas as pd
from pandas.testing import assert_frame_equal

from scd_type2.compare import compare_dfs

today = date.today().strftime("%d/%m/%Y")


def test_compare_dfs():
    df1 = pd.DataFrame({'A': [1, 2, 3, 3],
                        'B': [2, 3, 4, 4]})

    df2 = pd.DataFrame({'A': [1], 'B': [2]})

    response_start_df = pd.DataFrame.from_dict({'A': {0: 2, 1: 3, 2: 1},
                                                'B': {0: 3, 1: 4, 2: 2},
                                                'active': {0: 1, 1: 1, 2: 1},
                                                'start_date': {0: today, 1: today, 2: today}})
    response_end_df = pd.DataFrame.from_dict({'A': {0: 1}, 'B': {0: 2}, 'active': {0: 0}, 'end_date': {0: today}})

    idx = "A"
    start, end = compare_dfs(df1, df2, idx)
    assert_frame_equal(start, response_start_df)
    assert_frame_equal(end, response_end_df)

    idx = ["A"]
    start, end = compare_dfs(df1, df2, idx)
    assert_frame_equal(start, response_start_df)
    assert_frame_equal(end, response_end_df)

    idx = ["A", "B"]
    start, end = compare_dfs(df1, df2, idx)
    assert_frame_equal(start, response_start_df)
    assert_frame_equal(end, response_end_df)
